div { 
border: thin solid;
background-color: #edd;
margin: 1em;
 }

.site-nav { 
clear: both;
list-style-type: none;
display: inline;
margin-top: 0em;
padding-top: 0em;
 }

.site-nav a { 
float: left;
margin-left: 0.3em;
margin-right: 0.3em;
 }

.error { 
color: red; 
} 